@@ -43,7 +43,10 @@ class CreationConfig:
         lines = [
             "{} {}".format(self.base_resolved_python_sha256, self.base_resolved_python_path),
             "{} {:d} {:d} {:d}".format(
-                self.tox_version, self.sitepackages, self.usedevelop, self.alwayscopy,
+                self.tox_version,
+                self.sitepackages,
+                self.usedevelop,
+                self.alwayscopy,
             ),
         ]
         for dep in self.deps:
@@ -139,7 +142,9 @@ class VirtualEnv(object):
         if self._result_json_path is None:
             if self.envconfig.config.option.resultjson:
                 self._result_json_path = get_unique_file(
-                    self.path, PARALLEL_RESULT_JSON_PREFIX, PARALLEL_RESULT_JSON_SUFFIX,
+                    self.path,
+                    PARALLEL_RESULT_JSON_PREFIX,
+                    PARALLEL_RESULT_JSON_SUFFIX,
                 )
         return self._result_json_path
 
@@ -165,7 +170,7 @@ class VirtualEnv(object):
         return "<VirtualEnv at {!r}>".format(self.path)
 
     def getcommandpath(self, name, venv=True, cwd=None):
-        """ Return absolute path (str or localpath) for specified command name.
+        """Return absolute path (str or localpath) for specified command name.
 
         - If it's a local path we will rewrite it as as a relative path.
         - If venv is True we will check if the command is coming from the venv
@@ -214,7 +219,8 @@ class VirtualEnv(object):
                 "Maybe you forgot to specify a dependency? "
                 "See also the allowlist_externals envconfig setting.\n\n"
                 "DEPRECATION WARNING: this will be an error in tox 4 and above!".format(
-                    path, self.envconfig.envdir,
+                    path,
+                    self.envconfig.envdir,
                 ),
             )
 
@@ -239,8 +245,8 @@ class VirtualEnv(object):
         return False
 
     def update(self, action):
-        """ return status string for updating actual venv to match configuration.
-            if status string is empty, all is ok.
+        """return status string for updating actual venv to match configuration.
+        if status string is empty, all is ok.
         """
         rconfig = CreationConfig.readconfig(self.path_config)
         if self.envconfig.recreate:
@@ -320,10 +326,16 @@ class VirtualEnv(object):
         args = [self.envconfig.envpython, str(setup_py), "--name"]
         env = self._get_os_environ()
         output = action.popen(
-            args, cwd=setupdir, redirect=False, returnout=True, env=env, capture_err=False,
+            args,
+            cwd=setupdir,
+            redirect=False,
+            returnout=True,
+            env=env,
+            capture_err=False,
         )
         name = next(
-            (i for i in output.split("\n") if i and not i.startswith("pydev debugger:")), "",
+            (i for i in output.split("\n") if i and not i.startswith("pydev debugger:")),
+            "",
         )
         args = [
             self.envconfig.envpython,
@@ -418,6 +430,9 @@ class VirtualEnv(object):
                 redirect=reporter.verbosity() < reporter.Verbosity.DEBUG,
                 env=env,
             )
+        except KeyboardInterrupt:
+            self.status = "keyboardinterrupt"
+            raise
         finally:
             sys.stdout = old_stdout
 
@@ -481,7 +496,7 @@ class VirtualEnv(object):
             env = os.environ.copy()
 
         # in any case we honor per-testenv setenv configuration
-        env.update(self.envconfig.setenv)
+        env.update(self.envconfig.setenv.export())
 
         env["VIRTUAL_ENV"] = str(self.path)
         return env
@@ -511,7 +526,8 @@ class VirtualEnv(object):
                 # have to make strings as _pcall changes argv[0] to a local()
                 # happens if the same environment is invoked twice
                 message = "commands[{}] | {}".format(
-                    i, " ".join([pipes.quote(str(x)) for x in argv]),
+                    i,
+                    " ".join([pipes.quote(str(x)) for x in argv]),
                 )
                 action.setactivity(name, message)
                 # check to see if we need to ignore the return code
@@ -590,9 +606,14 @@ class VirtualEnv(object):
     def setupenv(self):
         if self.envconfig._missing_subs:
             self.status = (
-                "unresolvable substitution(s): {}. "
+                "unresolvable substitution(s):\n    {}\n"
                 "Environment variables are missing or defined recursively.".format(
-                    ",".join(["'{}'".format(m) for m in self.envconfig._missing_subs]),
+                    "\n    ".join(
+                        [
+                            "{}: '{}'".format(section_key, exc.name)
+                            for section_key, exc in sorted(self.envconfig._missing_subs.items())
+                        ],
+                    ),
                 )
             )
             return
@@ -618,6 +639,9 @@ class VirtualEnv(object):
                 status = e
                 if self.envconfig.config.option.skip_missing_interpreters == "true":
                     default_ret_code = 0
+            except KeyboardInterrupt:
+                self.status = "keyboardinterrupt"
+                raise
             if status:
                 str_status = str(status)
                 command_log = envlog.get_commandlog("setup")
@@ -682,6 +706,8 @@ def tox_testenv_create(venv, action):
         args.append("--always-copy")
     if not venv.envconfig.download:
         args.append("--no-download")
+    else:
+        args.append("--download")
     # add interpreter explicitly, to prevent using default (virtualenv.ini)
     args.extend(["--python", str(config_interpreter)])
 
